@@ -17,9 +17,9 @@ import Control.Monad.IO.Class (liftIO)
 import Data.Monoid
 import Data.Time.Calendar (Day)
 import Graphics.Vty (Event(..),Key(..))
-import Text.Megaparsec
+import Text.Megaparsec.Compat
 
-import Hledger.Cli hiding (progname,prognameandversion,green)
+import Hledger.Cli hiding (progname,prognameandversion)
 import Hledger.UI.UIOptions
 import Hledger.UI.UITypes
 import Hledger.UI.UIState
@@ -39,11 +39,11 @@ esInit _ _ ui@UIState{aScreen=ErrorScreen{}} = ui
 esInit _ _ _ = error "init function called with wrong screen type, should not happen"
 
 esDraw :: UIState -> [Widget Name]
-esDraw UIState{ --aopts=UIOpts{cliopts_=copts@CliOpts{}}
-               aScreen=ErrorScreen{..}
+esDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{}}
+              ,aScreen=ErrorScreen{..}
               ,aMode=mode} =
   case mode of
-    Help       -> [helpDialog, maincontent]
+    Help       -> [helpDialog copts, maincontent]
     -- Minibuffer e -> [minibuffer e, maincontent]
     _          -> [maincontent]
   where
@@ -105,7 +105,7 @@ esHandle _ _ = error "event handler called with wrong screen type, should not ha
 
 -- | Parse the file name, line and column number from a hledger parse error message, if possible.
 -- Temporary, we should keep the original parse error location. XXX
-hledgerparseerrorpositionp :: ParsecT Dec String t (String, Int, Int)
+hledgerparseerrorpositionp :: ParsecT MPErr String t (String, Int, Int)
 hledgerparseerrorpositionp = do
   anyChar `manyTill` char '"'
   f <- anyChar `manyTill` (oneOf ['"','\n'])
@@ -151,7 +151,7 @@ uiReloadJournalIfChanged copts d j ui = do
 -- are disabled, do nothing.
 uiCheckBalanceAssertions :: Day -> UIState -> UIState
 uiCheckBalanceAssertions d ui@UIState{aopts=UIOpts{cliopts_=copts}, ajournal=j}
-  | ignore_assertions_ copts = ui
+  | ignore_assertions_ $ inputopts_ copts = ui
   | otherwise =
     case journalCheckBalanceAssertions j of
       Right _  -> ui
