@@ -4,11 +4,12 @@
 
 module Hledger.UI.RegisterScreen
  (registerScreen
+ ,rsHandle
  ,rsSetAccount
+ ,rsCenterAndContinue
  )
 where
 
-import Lens.Micro.Platform ((^.))
 import Control.Monad
 import Control.Monad.IO.Class (liftIO)
 import Data.List
@@ -16,18 +17,20 @@ import Data.List.Split (splitOn)
 import Data.Monoid
 import Data.Maybe
 import qualified Data.Text as T
-import Data.Time.Calendar (Day)
+import Data.Time.Calendar
 import qualified Data.Vector as V
 import Graphics.Vty (Event(..),Key(..),Modifier(..))
 import Brick
 import Brick.Widgets.List
 import Brick.Widgets.Edit
 import Brick.Widgets.Border (borderAttr)
+import Lens.Micro.Platform
+import Safe
 import System.Console.ANSI
 
 
 import Hledger
-import Hledger.Cli hiding (progname,prognameandversion,green)
+import Hledger.Cli hiding (progname,prognameandversion)
 import Hledger.UI.UIOptions
 -- import Hledger.UI.Theme
 import Hledger.UI.UITypes
@@ -76,6 +79,7 @@ rsInit d reset ui@UIState{aopts=UIOpts{cliopts_=CliOpts{reportopts_=ropts}}, ajo
       where
         displayitem (t, _, _issplit, otheracctsstr, change, bal) =
           RegisterScreenItem{rsItemDate          = showDate $ transactionRegisterDate q thisacctq t
+                            ,rsItemStatus        = tstatus t
                             ,rsItemDescription   = T.unpack $ tdescription t
                             ,rsItemOtherAccounts = case splitOn ", " otheracctsstr of
                                                      [s] -> s
@@ -85,20 +89,41 @@ rsInit d reset ui@UIState{aopts=UIOpts{cliopts_=CliOpts{reportopts_=ropts}}, ajo
                             ,rsItemBalanceAmount = showMixedAmountOneLineWithoutPrice bal
                             ,rsItemTransaction   = t
                             }
-
+    -- blank items are added to allow more control of scroll position; we won't allow movement over these
+    blankitems = replicate 100  -- 100 ought to be enough for anyone
+          RegisterScreenItem{rsItemDate          = ""
+                            ,rsItemStatus        = Unmarked
+                            ,rsItemDescription   = ""
+                            ,rsItemOtherAccounts = ""
+                            ,rsItemChangeAmount  = ""
+                            ,rsItemBalanceAmount = ""
+                            ,rsItemTransaction   = nulltransaction
+                            }
     -- build the List
-    newitems = list RegisterList (V.fromList displayitems) 1
+    newitems = list RegisterList (V.fromList $ displayitems ++ blankitems) 1
 
-    -- keep the selection on the previously selected transaction if possible,
-    -- (eg after toggling nonzero mode), otherwise select the last element.
+    -- decide which transaction is selected:
+    -- if reset is true, the last (latest) transaction;
+    -- otherwise, the previously selected transaction if possible;
+    -- otherwise, the transaction nearest in date to it;
+    -- or if there's several with the same date, the nearest in journal order;
+    -- otherwise, the last (latest) transaction. 
     newitems' = listMoveTo newselidx newitems
       where
-        newselidx = case (reset, listSelectedElement rsList) of
-                      (True, _)    -> endidx
-                      (_, Nothing) -> endidx
-                      (_, Just (_,RegisterScreenItem{rsItemTransaction=Transaction{tindex=ti}}))
-                                   -> fromMaybe endidx $ findIndex ((==ti) . tindex . rsItemTransaction) displayitems
-        endidx = length displayitems
+        newselidx = 
+          case (reset, listSelectedElement rsList) of
+            (True, _)    -> endidx
+            (_, Nothing) -> endidx
+            (_, Just (_, RegisterScreenItem{rsItemTransaction=Transaction{tindex=prevselidx, tdate=prevseld}})) ->
+              headDef endidx $ catMaybes [
+                 findIndex ((==prevselidx) . tindex . rsItemTransaction) displayitems
+                ,findIndex ((==nearestidbydatethenid) . Just . tindex . rsItemTransaction) displayitems
+                ]
+              where
+                nearestidbydatethenid = third3 <$> (headMay $ sort
+                  [(abs $ diffDays (tdate t) prevseld, abs (tindex t - prevselidx), tindex t) | t <- ts])
+                ts = map rsItemTransaction displayitems
+        endidx = length displayitems - 1
 
 rsInit _ _ _ = error "init function called with wrong screen type, should not happen"
 
@@ -108,7 +133,7 @@ rsDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
                             ,aMode=mode
                             } =
   case mode of
-    Help       -> [helpDialog, maincontent]
+    Help       -> [helpDialog copts, maincontent]
     -- Minibuffer e -> [minibuffer e, maincontent]
     _          -> [maincontent]
   where
@@ -127,7 +152,7 @@ rsDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
         -- columns and whitespace.  If they don't get all they need,
         -- allocate it to them proportionally to their maximum widths.
         whitespacewidth = 10 -- inter-column whitespace, fixed width
-        minnonamtcolswidth = datewidth + 2 + 2 -- date column plus at least 2 for desc and accts
+        minnonamtcolswidth = datewidth + 1 + 2 + 2 -- date column plus at least 1 for status and 2 for desc and accts
         maxamtswidth = max 0 (totalwidth - minnonamtcolswidth - whitespacewidth)
         maxchangewidthseen = maximum' $ map (strWidth . rsItemChangeAmount) displayitems
         maxbalwidthseen = maximum' $ map (strWidth . rsItemBalanceAmount) displayitems
@@ -140,7 +165,7 @@ rsDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
         -- maxdescacctswidth = totalwidth - (whitespacewidth - 4) - changewidth - balwidth
         maxdescacctswidth =
           -- trace (show (totalwidth, datewidth, changewidth, balwidth, whitespacewidth)) $
-          max 0 (totalwidth - datewidth - changewidth - balwidth - whitespacewidth)
+          max 0 (totalwidth - datewidth - 1 - changewidth - balwidth - whitespacewidth)
         -- allocating proportionally.
         -- descwidth' = maximum' $ map (strWidth . second6) displayitems
         -- acctswidth' = maximum' $ map (strWidth . third6) displayitems
@@ -174,11 +199,11 @@ rsDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
           <+> str "/"
           <+> total
           <+> str ")"
-          <+> (if ignore_assertions_ copts then withAttr (borderAttr <> "query") (str " ignoring balance assertions") else str "")
+          <+> (if ignore_assertions_ $ inputopts_ copts then withAttr (borderAttr <> "query") (str " ignoring balance assertions") else str "")
           where
             togglefilters =
               case concat [
-                   uiShowClearedStatus $ clearedstatus_ ropts
+                   uiShowStatus copts $ statuses_ ropts
                   ,if real_ ropts then ["real"] else []
                   ,if empty_ ropts then [] else ["nonzero"]
                   ] of
@@ -187,7 +212,8 @@ rsDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
             cur = str $ case rsList ^. listSelectedL of
                          Nothing -> "-"
                          Just i -> show (i + 1)
-            total = str $ show $ length displayitems
+            total = str $ show $ length nonblanks
+            nonblanks = V.takeWhile (not . null . rsItemDate) $ rsList^.listElementsL
 
             -- query = query_ $ reportopts_ $ cliopts_ opts
 
@@ -220,7 +246,9 @@ rsDrawItem (datewidth,descwidth,acctswidth,changewidth,balwidth) selected Regist
   Widget Greedy Fixed $ do
     render $
       str (fitString (Just datewidth) (Just datewidth) True True rsItemDate) <+>
-      str "  " <+>
+      str " " <+>
+      str (fitString (Just 1) (Just 1) True True (show rsItemStatus)) <+>
+      str " " <+>
       str (fitString (Just descwidth) (Just descwidth) True True rsItemDescription) <+>
       str "  " <+>
       str (fitString (Just acctswidth) (Just acctswidth) True True rsItemOtherAccounts) <+>
@@ -244,7 +272,11 @@ rsHandle ui@UIState{
   ,aMode=mode
   } ev = do
   d <- liftIO getCurrentDay
-
+  let 
+    journalspan = journalDateSpan False j
+    nonblanks = V.takeWhile (not . null . rsItemDate) $ rsList^.listElementsL
+    lastnonblankidx = max 0 (length nonblanks - 1)
+  
   case mode of
     Minibuffer ed ->
       case ev of
@@ -285,24 +317,31 @@ rsHandle ui@UIState{
                           rsItemTransaction=Transaction{tsourcepos=GenericSourcePos f l c}}) -> (Just (l, Just c),f)
                         Just (_, RegisterScreenItem{
                           rsItemTransaction=Transaction{tsourcepos=JournalSourcePos f (l,_)}}) -> (Just (l, Nothing),f)
-        VtyEvent (EvKey (KChar 'H') []) -> continue $ regenerateScreens j d $ toggleHistorical ui
-        VtyEvent (EvKey (KChar 'F') []) -> scrollTop >> (continue $ regenerateScreens j d $ toggleFlat ui)
-        VtyEvent (EvKey (KChar 'Z') []) -> scrollTop >> (continue $ regenerateScreens j d $ toggleEmpty ui)
-        VtyEvent (EvKey (KChar 'C') []) -> scrollTop >> (continue $ regenerateScreens j d $ toggleCleared ui)
-        VtyEvent (EvKey (KChar 'U') []) -> scrollTop >> (continue $ regenerateScreens j d $ toggleUncleared ui)
-        VtyEvent (EvKey (KChar 'R') []) -> scrollTop >> (continue $ regenerateScreens j d $ toggleReal ui)
-        VtyEvent (EvKey (KChar '/') []) -> (continue $ regenerateScreens j d $ showMinibuffer ui)
+
+        -- display mode/query toggles
+        VtyEvent (EvKey (KChar 'H') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleHistorical ui
+        VtyEvent (EvKey (KChar 'F') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleFlat ui
+        VtyEvent (EvKey (KChar 'Z') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleEmpty ui
+        VtyEvent (EvKey (KChar 'R') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleReal ui
+        VtyEvent (EvKey (KChar 'U') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleUnmarked ui
+        VtyEvent (EvKey (KChar 'P') []) -> rsCenterAndContinue $ regenerateScreens j d $ togglePending ui
+        VtyEvent (EvKey (KChar 'C') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleCleared ui
+
+        VtyEvent (EvKey (KChar '/') []) -> continue $ regenerateScreens j d $ showMinibuffer ui
         VtyEvent (EvKey (KDown)     [MShift]) -> continue $ regenerateScreens j d $ shrinkReportPeriod d ui
         VtyEvent (EvKey (KUp)       [MShift]) -> continue $ regenerateScreens j d $ growReportPeriod d ui
         VtyEvent (EvKey (KRight)    [MShift]) -> continue $ regenerateScreens j d $ nextReportPeriod journalspan ui
         VtyEvent (EvKey (KLeft)     [MShift]) -> continue $ regenerateScreens j d $ previousReportPeriod journalspan ui
         VtyEvent (EvKey k           []) | k `elem` [KBS, KDel] -> (continue $ regenerateScreens j d $ resetFilter ui)
-        VtyEvent (EvKey k           []) | k `elem` [KLeft, KChar 'h']  -> continue $ popScreen ui
-        VtyEvent (EvKey k           []) | k `elem` [KRight, KChar 'l'] -> do
+        VtyEvent e | e `elem` moveLeftEvents  -> continue $ popScreen ui
+        VtyEvent (EvKey (KChar 'l') [MCtrl]) -> scrollSelectionToMiddle rsList >> invalidateCache >> continue ui
+
+        -- enter transaction screen for selected transaction
+        VtyEvent e | e `elem` moveRightEvents -> do
           case listSelectedElement rsList of
             Just (_, RegisterScreenItem{rsItemTransaction=t}) ->
               let
-                ts = map rsItemTransaction $ V.toList $ listElements rsList
+                ts = map rsItemTransaction $ V.toList $ nonblanks
                 numberedts = zip [1..] ts
                 i = fromIntegral $ maybe 0 (+1) $ elemIndex t ts -- XXX
               in
@@ -310,21 +349,40 @@ rsHandle ui@UIState{
                                                           ,tsTransactions=numberedts
                                                           ,tsAccount=rsAccount} ui
             Nothing -> continue ui
-        -- fall through to the list's event handler (handles [pg]up/down)
+
+        -- prevent moving down over blank padding items;
+        -- instead scroll down by one, until maximally scrolled - shows the end has been reached
+        VtyEvent e | e `elem` moveDownEvents, isBlankElement mnextelement -> do
+          vScrollBy (viewportScroll $ rsList^.listNameL) 1 
+          continue ui
+          where 
+            mnextelement = listSelectedElement $ listMoveDown rsList
+
+        -- if page down or end leads to a blank padding item, stop at last non-blank
+        VtyEvent e@(EvKey k           []) | k `elem` [KPageDown, KEnd] -> do
+          list <- handleListEvent e rsList
+          if isBlankElement $ listSelectedElement list
+          then do
+            let list' = listMoveTo lastnonblankidx list
+            scrollSelectionToMiddle list'
+            continue ui{aScreen=s{rsList=list'}}
+          else
+            continue ui{aScreen=s{rsList=list}}
+          
+        -- fall through to the list's event handler (handles other [pg]up/down events)
         VtyEvent ev -> do
-                let ev' = case ev of
-                            EvKey (KChar 'k') [] -> EvKey (KUp) []
-                            EvKey (KChar 'j') [] -> EvKey (KDown) []
-                            _                    -> ev
-                newitems <- handleListEvent ev' rsList
-                continue ui{aScreen=s{rsList=newitems}}
-                -- continue =<< handleEventLensed ui someLens ev
+          let ev' = normaliseMovementKeys ev
+          newitems <- handleListEvent ev' rsList
+          continue ui{aScreen=s{rsList=newitems}}
+
         AppEvent _        -> continue ui
         MouseDown _ _ _ _ -> continue ui
         MouseUp _ _ _     -> continue ui
-      where
-        -- Encourage a more stable scroll position when toggling list items (cf AccountsScreen.hs)
-        scrollTop = vScrollToBeginning $ viewportScroll RegisterViewport
-        journalspan = journalDateSpan False j
 
 rsHandle _ _ = error "event handler called with wrong screen type, should not happen"
+
+isBlankElement mel = ((rsItemDate . snd) <$> mel) == Just "" 
+
+rsCenterAndContinue ui = do
+  scrollSelectionToMiddle $ rsList $ aScreen ui
+  continue ui
