@@ -10,6 +10,7 @@ import Data.Default
 import Data.Functor.Compat ((<$>))
 #endif
 import Data.List (intercalate)
+import System.Environment
 
 import Hledger.Cli hiding (progname,version,prognameandversion)
 import Hledger.UI.Theme (themeNames)
@@ -39,6 +40,15 @@ uiflags = [
   -- ,flagReq ["drop"] (\s opts -> Right $ setopt "drop" s opts) "N" "with --flat, omit this many leading account name components"
   -- ,flagReq  ["format"] (\s opts -> Right $ setopt "format" s opts) "FORMATSTR" "use this custom line format"
   -- ,flagNone ["no-elide"] (\opts -> setboolopt "no-elide" opts) "don't compress empty parent accounts on one line"
+  ,flagReq  ["status-toggles"] (\s opts -> Right $ setopt "status-toggles" s opts) "N"
+    (intercalate "\n"
+      ["choose how status toggles work:"
+      ," 1 UPC toggles X/all"
+      ," 2 UPC cycles X/not-X/all"
+      ," 3 UPC toggles each X"
+--      ," 4 upc sets X, UPC sets not-X"
+--      ," 5 upc toggles X, UPC toggles not-X"
+      ])
  ]
 
 --uimode :: Mode [([Char], [Char])]
@@ -85,6 +95,12 @@ checkUIOpts opts =
       Just t | not $ elem t themeNames -> Left $ "invalid theme name: "++t
       _                                -> Right ()
 
+-- XXX some refactoring seems due
 getHledgerUIOpts :: IO UIOpts
-getHledgerUIOpts = processArgs uimode >>= return . decodeRawOpts >>= rawOptsToUIOpts
+--getHledgerUIOpts = processArgs uimode >>= return . decodeRawOpts >>= rawOptsToUIOpts
+getHledgerUIOpts = do
+  args <- getArgs >>= expandArgsAt
+  let args' = replaceNumericFlags args 
+  let cmdargopts = either usageError id $ process uimode args'
+  rawOptsToUIOpts $ decodeRawOpts cmdargopts 
 
