@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {- | Rendering & misc. helpers. -}
 
 {-# LANGUAGE OverloadedStrings #-}
@@ -11,33 +12,27 @@ import Brick.Widgets.Border.Style
 -- import Brick.Widgets.Center
 import Brick.Widgets.Dialog
 import Brick.Widgets.Edit
+import Brick.Widgets.List
+import Control.Monad.IO.Class
 import Data.List
+import Data.Maybe
 import Data.Monoid
-import Graphics.Vty (Event(..),Key(..),Color,Attr,currentAttr)
+import Graphics.Vty (Event(..),Key(..),Modifier(..),Color,Attr,currentAttr)
 import Lens.Micro.Platform
-import System.Process
+import System.Environment
 
-import Hledger
+import Hledger hiding (Color)
+import Hledger.Cli (CliOpts(rawopts_))
+import Hledger.Cli.DocFiles
 import Hledger.UI.UITypes
 import Hledger.UI.UIState
 
 
-runInfo = runCommand "hledger-ui --info" >>= waitForProcess
-runMan  = runCommand "hledger-ui --man" >>= waitForProcess
-runHelp = runCommand "hledger-ui --help | less" >>= waitForProcess
-
 -- ui
 
-uiShowClearedStatus mc =
- case mc of
-   Just Cleared   -> ["cleared"]
-   Just Pending   -> ["pending"]
-   Just Uncleared -> ["uncleared"]
-   Nothing        -> []
-
 -- | Draw the help dialog, called when help mode is active.
-helpDialog :: Widget Name
-helpDialog =
+helpDialog :: CliOpts -> Widget Name
+helpDialog copts =
   Widget Fixed Fixed $ do
     c <- getContext
     render $
@@ -48,10 +43,11 @@ helpDialog =
               padLeftRight 1 $
                 vBox [
                    str "NAVIGATION"
-                  ,renderKey ("UP/DOWN/k/j/PGUP/PGDN/HOME/END", "")
+                  ,renderKey ("UP/DOWN/PGUP/PGDN/HOME/END", "")
                   ,str "  move selection"
-                  ,renderKey ("RIGHT/l", "more detail")
-                  ,renderKey ("LEFT/h", "previous screen")
+                  ,renderKey ("RIGHT", "more detail")
+                  ,renderKey ("LEFT", "previous screen")
+                  ,str "  (or vi/emacs movement keys)"
                   ,renderKey ("ESC", "cancel / reset to top")
                   ,str " "
                   ,str "MISC"
@@ -59,14 +55,15 @@ helpDialog =
                   ,renderKey ("a", "add transaction (hledger add)")
                   ,renderKey ("A", "add transaction (hledger-iadd)")
                   ,renderKey ("E", "open editor")
-                  ,renderKey ("g", "reload data")
                   ,renderKey ("I", "toggle balance assertions")
+                  ,renderKey ("CTRL-l", "redraw & recenter")
+                  ,renderKey ("g", "reload data")
                   ,renderKey ("q", "quit")
                   ,str " "
                   ,str "MANUAL"
-                  ,str "from help dialog:"
-                  ,renderKey ("t", "text")
-                  ,renderKey ("m", "man page")
+                  ,str "choose format:"
+                  ,renderKey ("p", "pager")
+                  ,renderKey ("m", "man")
                   ,renderKey ("i", "info")
                 ]
              ,padLeftRight 1 $
@@ -77,8 +74,21 @@ helpDialog =
                   ,renderKey ("t", "set report period to today")
                   ,str " "
                   ,renderKey ("/", "set a filter query")
-                  ,renderKey ("C", "toggle cleared/all")
-                  ,renderKey ("U", "toggle uncleared/all")
+                  ,renderKey ("U", 
+                    ["toggle unmarked/all"
+                    ,"cycle unmarked/not unmarked/all"
+                    ,"toggle unmarked filter"
+                    ] !! (statusstyle-1))
+                  ,renderKey ("P",
+                    ["toggle pending/all"
+                    ,"cycle pending/not pending/all"
+                    ,"toggle pending filter"
+                    ] !! (statusstyle-1))
+                  ,renderKey ("C",
+                    ["toggle cleared/all"
+                    ,"cycle cleared/not cleared/all"
+                    ,"toggle cleared filter"
+                    ] !! (statusstyle-1))
                   ,renderKey ("R", "toggle real/all")
                   ,renderKey ("Z", "toggle nonzero/all")
                   ,renderKey ("DEL/BS", "remove filters")
@@ -109,15 +119,18 @@ helpDialog =
           ]
   where
     renderKey (key,desc) = withAttr (borderAttr <> "keys") (str key) <+> str " " <+> str desc
+    statusstyle = min 3 $ fromMaybe 1 $ maybeintopt "status-toggles" $ rawopts_ copts 
 
 -- | Event handler used when help mode is active.
+-- May invoke $PAGER, less, man or info, which is likely to fail on MS Windows, TODO.
 helpHandle :: UIState -> BrickEvent Name AppEvent -> EventM Name (Next UIState)
-helpHandle ui ev =
+helpHandle ui ev = do
+  pagerprog <- liftIO $ fromMaybe "less" <$> lookupEnv "PAGER"
   case ev of
-    VtyEvent (EvKey k []) | k `elem` [KEsc, KLeft, KChar 'h', KChar '?'] -> continue $ setMode Normal ui
-    VtyEvent (EvKey (KChar 't') []) -> suspendAndResume $ runHelp >> return ui'
-    VtyEvent (EvKey (KChar 'm') []) -> suspendAndResume $ runMan  >> return ui'
-    VtyEvent (EvKey (KChar 'i') []) -> suspendAndResume $ runInfo >> return ui'
+    VtyEvent e | e `elem` (moveLeftEvents ++ [EvKey KEsc [], EvKey (KChar '?') []]) -> continue $ setMode Normal ui
+    VtyEvent (EvKey (KChar 'p') []) -> suspendAndResume $ runPagerForTopic pagerprog "hledger-ui" >> return ui'
+    VtyEvent (EvKey (KChar 'm') []) -> suspendAndResume $ runManForTopic             "hledger-ui" >> return ui'
+    VtyEvent (EvKey (KChar 'i') []) -> suspendAndResume $ runInfoForTopic            "hledger-ui" >> return ui'
     _ -> continue ui
   where
     ui' = setMode Normal ui
@@ -127,7 +140,11 @@ minibuffer :: Editor String Name -> Widget Name
 minibuffer ed =
   forceAttr (borderAttr <> "minibuffer") $
   hBox $
+#if MIN_VERSION_brick(0,19,0)
+  [txt "filter: ", renderEditor (str . unlines) True ed]
+#else
   [txt "filter: ", renderEditor True ed]
+#endif
 
 -- | Wrap a widget in the default hledger-ui screen layout.
 defaultLayout :: Widget Name -> Widget Name -> Widget Name -> Widget Name
@@ -246,3 +263,44 @@ margin h v mcolour = \w ->
 withBorderAttr :: Attr -> Widget Name -> Widget Name
 withBorderAttr attr = updateAttrMap (applyAttrMappings [(borderAttr, attr)])
 
+-- | Like brick's continue, but first run some action to modify brick's state.
+-- This action does not affect the app state, but might eg adjust a widget's scroll position.
+continueWith :: EventM n () -> ui -> EventM n (Next ui)
+continueWith brickaction ui = brickaction >> continue ui
+
+-- | Scroll a list's viewport so that the selected item is centered in the
+-- middle of the display area.
+scrollToTop :: List Name e -> EventM Name ()
+scrollToTop list = do
+  let vpname = list^.listNameL
+  setTop (viewportScroll vpname) 0 
+
+-- | Scroll a list's viewport so that the selected item is centered in the
+-- middle of the display area.
+scrollSelectionToMiddle :: List Name e -> EventM Name ()
+scrollSelectionToMiddle list = do
+  let mselectedrow = list^.listSelectedL 
+      vpname = list^.listNameL
+  mvp <- lookupViewport vpname
+  case (mselectedrow, mvp) of
+    (Just selectedrow, Just vp) -> do
+      let
+        itemheight   = dbg4 "itemheight" $ list^.listItemHeightL
+        vpheight     = dbg4 "vpheight" $ vp^.vpSize._2
+        itemsperpage = dbg4 "itemsperpage" $ vpheight `div` itemheight
+        toprow       = dbg4 "toprow" $ max 0 (selectedrow - (itemsperpage `div` 2)) -- assuming ViewportScroll's row offset is measured in list items not screen rows
+      setTop (viewportScroll vpname) toprow 
+    _ -> return ()
+
+--                 arrow keys       vi keys               emacs keys
+moveUpEvents    = [EvKey KUp []   , EvKey (KChar 'k') [], EvKey (KChar 'p') [MCtrl]]
+moveDownEvents  = [EvKey KDown [] , EvKey (KChar 'j') [], EvKey (KChar 'n') [MCtrl]]
+moveLeftEvents  = [EvKey KLeft [] , EvKey (KChar 'h') [], EvKey (KChar 'b') [MCtrl]]
+moveRightEvents = [EvKey KRight [], EvKey (KChar 'l') [], EvKey (KChar 'f') [MCtrl]]
+
+normaliseMovementKeys ev
+  | ev `elem` moveUpEvents    = EvKey KUp []
+  | ev `elem` moveDownEvents  = EvKey KDown []
+  | ev `elem` moveLeftEvents  = EvKey KLeft []
+  | ev `elem` moveRightEvents = EvKey KRight []
+  | otherwise = ev
